// Document window handling
// Copyright (C) 2009 Fred Allen.

// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

#include "includes.h"
#import <AppKit/AppKit.h>

@interface ContentView : NSView
@end

@implementation ContentView
{
}

- (id)initWithFrame:(NSRect)frame
{
	self = [super initWithFrame:frame];
	if (self)
	{
	}
	return self;
}

- (BOOL)isFlipped
{
	return true;				// window coords are flipped
}

@end


WINDOWLISTELEMENT* LocateEditorWindow(NSWindow* window)
// Passed in an OS X Window Ref, see if one of our window elements is this window
// and return it, else return NULL
{
	WINDOWLISTELEMENT* retWindowElement;
	WINDOWLISTELEMENT* curWindowElement;

	retWindowElement = NULL;
	curWindowElement = g_windowListHead;
	while (curWindowElement && retWindowElement == NULL)
	{
		if (curWindowElement->windowRef == window)
		{
			retWindowElement = curWindowElement;
		}
		curWindowElement = curWindowElement->nextElement;
	}
	return (retWindowElement);
}

WINDOWLISTELEMENT* LocateEditorWindow(NSScroller* scroller)
// Passed in an OS X Scroller Ref, see if one of our window elements is this window
// and return it, else return NULL
{
	WINDOWLISTELEMENT* retWindowElement;
	WINDOWLISTELEMENT* curWindowElement;

	retWindowElement = NULL;
	curWindowElement = g_windowListHead;
	while (curWindowElement && retWindowElement == NULL)
	{
		if (curWindowElement->vScrollBarViewRef == scroller || curWindowElement->hScrollBarViewRef == scroller)
		{
			retWindowElement = curWindowElement;
		}
		curWindowElement = curWindowElement->nextElement;
	}
	return (retWindowElement);
}


@interface WindowDelegateHandlerObject : NSObject <NSWindowDelegate>
@end

@implementation WindowDelegateHandlerObject
{
}

- (id)init
{
	if (self = [super init])
	{
	}
	return self;
}

- (BOOL)windowShouldClose:(NSWindow*)window
{
	WINDOWLISTELEMENT* theWindowElement = LocateEditorWindow(window);

	if (theWindowElement)
	{
		EDITOR_WINDOW* theWindow = theWindowElement->theEditorWindow;
		HandleShellCommand("close", 1, (const char**) & (theWindow->buffer->contentName));
		return false;			// we handled the window closing
	}
	return true;
}

- (void)windowDidBecomeKey:(NSNotification*)notification
{
	NSWindow* window = (NSWindow*)notification.object;
	WINDOWLISTELEMENT* theWindowElement = LocateEditorWindow(window);

	if (theWindowElement)
	{
		EDITOR_WINDOW* theWindow = theWindowElement->theEditorWindow;
		EDITOR_VIEW* theView = (EDITOR_VIEW*)theWindow->windowInfo;
		GUI_VIEW_INFO* theViewInfo = (GUI_VIEW_INFO*)theView->viewInfo;

		[window invalidateCursorRectsForView:theWindowElement->viewRef];

		ViewStartSelectionChange(theView);
		theView->active = true;
		ViewEndSelectionChange(theView);

		theViewInfo->bounds.x = 0;
		theViewInfo->bounds.y = 0;
		theViewInfo->bounds.w = theWindowElement->viewRef.frame.size.width;
		theViewInfo->bounds.h = theWindowElement->viewRef.frame.size.height;
		RecalculateViewFontInfo(theView);
		AdjustDocumentWindowScrollThumbs(theView);
		theWindowElement->viewRef.needsDisplay = YES;
	}
}

- (void)windowDidResignKey:(NSNotification*)notification
{
	NSWindow* window = (NSWindow*)notification.object;
	WINDOWLISTELEMENT* theWindowElement = LocateEditorWindow(window);
	if (theWindowElement)
	{
		EDITOR_WINDOW* theWindow = theWindowElement->theEditorWindow;
		EDITOR_VIEW* theView = (EDITOR_VIEW*)theWindow->windowInfo;

		[window invalidateCursorRectsForView:theWindowElement->viewRef];
		ViewStartSelectionChange(theView);
		theView->active = false;
		ViewEndSelectionChange(theView);
	}
}

- (void)windowDidUpdate:(NSNotification *)notification
{
	NSWindow* window = (NSWindow*)notification.object;
	WINDOWLISTELEMENT* theWindowElement = LocateEditorWindow(window);

	if (theWindowElement)
	{
		if (!theWindowElement->windowInitialized)
		{
			theWindowElement->windowInitialized = true;
			[self windowDidBecomeKey:notification];
		}
	}
}

- (void)windowDidBecomeMain:(NSNotification*)notification
{
	NSWindow* window = (NSWindow*)notification.object;
	WINDOWLISTELEMENT* theWindowElement = LocateEditorWindow(window);

	if (theWindowElement)
	{
		UnlinkWindowElement(theWindowElement);									// remove this window from the list
		LinkWindowElement(theWindowElement);									// put it back on the list as the top one
	}
}

- (void)windowDidResize:(NSNotification*)notification
{
	NSWindow* window = (NSWindow*)notification.object;
	WINDOWLISTELEMENT* theWindowElement = LocateEditorWindow(window);

	if (theWindowElement)
	{
		EDITOR_WINDOW* theWindow = theWindowElement->theEditorWindow;
		EDITOR_VIEW* theView = (EDITOR_VIEW*)theWindow->windowInfo;
		GUI_VIEW_INFO* theViewInfo = (GUI_VIEW_INFO*)theView->viewInfo;

		[window invalidateCursorRectsForView:theWindowElement->viewRef];
		theViewInfo->bounds.x = 0;
		theViewInfo->bounds.y = 0;
		theViewInfo->bounds.w = theWindowElement->viewRef.frame.size.width;
		theViewInfo->bounds.h = theWindowElement->viewRef.frame.size.height;
		RecalculateViewFontInfo(theView);
		theWindowElement->viewRef.needsDisplay = YES;
	}
}

- (NSSize)windowWillResize:(NSWindow*)window toSize:(NSSize)frameSize
{
	if (frameSize.width < 360)
	{
		frameSize.width = 360;
	}
	if (frameSize.height < 200)
	{
		frameSize.height = 200;
	}
	return frameSize;
}

- (void)scrollerAction:(NSScroller*)scroller
{
	WINDOWLISTELEMENT* theWindowElement = LocateEditorWindow(scroller);

	if (theWindowElement)
	{
		EDITOR_WINDOW* theWindow = theWindowElement->theEditorWindow;
		EDITOR_VIEW* theView = (EDITOR_VIEW*)theWindow->windowInfo;

		switch (scroller.hitPart)
		{
			case NSScrollerKnob:
			{
				if (scroller == theWindowElement->vScrollBarViewRef)
				{
					ScrollEditViewThumbVerticalEvent(theView, scroller.doubleValue * (double)theView->parentBuffer->textUniverse->totalLines);
				}
				else
				{
					ScrollEditViewThumbHorizontalEvent(theView, scroller.doubleValue * (double)EDIT_MAX_WIDTH);
				}
				break;
			}
			case NSScrollerDecrementPage:
			{
				if (scroller == theWindowElement->vScrollBarViewRef)
				{
					ScrollEditViewEvent(theView, VVK_DOCUMENTPAGEUP, false);
				}
				else
				{
					ScrollEditViewEvent(theView, VVK_DOCUMENTPAGELEFT, false);
				}
				break;
			}
			case NSScrollerIncrementPage:
			{
				if (scroller == theWindowElement->vScrollBarViewRef)
				{
					ScrollEditViewEvent(theView, VVK_DOCUMENTPAGEDOWN, false);
				}
				else
				{
					ScrollEditViewEvent(theView, VVK_DOCUMENTPAGERIGHT, false);
				}
				break;
			}
		}
	}
}
@end

static WindowDelegateHandlerObject* windowDelegateHandler;

void LinkWindowElement(WINDOWLISTELEMENT* theElement)
// passed a WINDOWLISTELEMENT, link it to the start of the global list
{
	if (g_windowListHead)									// point previous back to new
	{
		g_windowListHead->previousElement = theElement;
	}
	theElement->nextElement = g_windowListHead;				// point to the old head of the list
	theElement->previousElement = NULL;						// no previous element
	g_windowListHead = theElement;							// point the head at the new element
}

void UnlinkWindowElement(WINDOWLISTELEMENT* theElement)
// passed a WINDOWLISTELEMENT, unlink it from the global list
{
	if (theElement->nextElement)
	{
		theElement->nextElement->previousElement = theElement->previousElement;
	}
	if (theElement->previousElement)
	{
		theElement->previousElement->nextElement = theElement->nextElement;
	}
	else
	{
		g_windowListHead = theElement->nextElement;			// this one was at the head, so update
	}
}

WINDOWLISTELEMENT* LocateActiveEditorWindow()
// Go through the window list and return the first window that is active (should be the only one)
// and return it, else return NULL if no active window found
// This checks to see if the window is active from OS X point of view, instead of checking to see if the view is active
{
	WINDOWLISTELEMENT* retWindowElement;
	WINDOWLISTELEMENT* curWindowElement;

	retWindowElement = NULL;
	curWindowElement = g_windowListHead;
	while (curWindowElement && retWindowElement == NULL)
	{
		if (curWindowElement->windowRef.keyWindow)
		{
			retWindowElement = curWindowElement;
		}
		curWindowElement = curWindowElement->nextElement;
	}
	return (retWindowElement);
}

void InvalidateAllBottomBars()
// Invalidate all the bottom bars for all the windows
{
	WINDOWLISTELEMENT* curWindowElement;

	curWindowElement = g_windowListHead;
	while (curWindowElement)
	{
		curWindowElement->toolBarRef.needsDisplay = YES;
		curWindowElement = curWindowElement->nextElement;
	}
}

static bool SetDocumentForegroundColor(EDITOR_WINDOW* theWindow, EDITOR_COLOR foreground)
// set the foreground color used to draw theWindow
{
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theViewInfo = theView->viewInfo;
	theViewInfo->foregroundColor = foreground;
	return (true);
}

static bool SetDocumentBackgroundColor(EDITOR_WINDOW* theWindow, EDITOR_COLOR background)
// set the background color used to draw theWindow
{
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theViewInfo = theView->viewInfo;
	theViewInfo->backgroundColor = background;
	return (true);
}

void AdjustDocumentWindowStatus(EDITOR_WINDOW* theWindow)
// The x or y position of the text cursor has changed, or some other status, invalidate the status area to be redrawn.
{
	WINDOWLISTELEMENT* theWindowElement;
	EDITOR_VIEW* theView;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;
	theWindowElement->statusRef.needsDisplay = YES;
}

void AdjustDocumentWindowViewStatus(EDITOR_VIEW* theView)
// adjust the status bar of the document window with the given view
{
	AdjustDocumentWindowStatus(theView->parentWindow);
}

static void AdjustDocumentWindowScrollThumbsAndStatus(EDITOR_VIEW* theView)
// adjust the parts of the window that change when the view changes
{
	AdjustDocumentWindowScrollThumbs(theView);
	AdjustDocumentWindowViewStatus(theView);
}

void AdjustDocumentWindowScrollThumbs(EDITOR_VIEW* theView)
// look at the data of the passed view, and adjust the thumbs of its scroll bars so that they are correct
{
	GUI_VIEW_INFO* theViewInfo;
	WINDOWLISTELEMENT* theWindowElement;

	theViewInfo = theView->viewInfo;
	theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;

	if (theView->parentBuffer->textUniverse->totalLines)
	{
		[theWindowElement->vScrollBarViewRef setKnobProportion:(double)theViewInfo->numLines / (double)theView->parentBuffer->textUniverse->totalLines];
		[theWindowElement->vScrollBarViewRef setDoubleValue:(double)theViewInfo->topLine / (double)theView->parentBuffer->textUniverse->totalLines];
	}
	else
	{
		[theWindowElement->vScrollBarViewRef setKnobProportion:1.0];
		[theWindowElement->vScrollBarViewRef setDoubleValue:1.0];
	}

	[theWindowElement->hScrollBarViewRef setKnobProportion:(double)theViewInfo->bounds.w / (double)EDIT_MAX_WIDTH];
	[theWindowElement->hScrollBarViewRef setDoubleValue:(double)theViewInfo->leftPixel / (double)EDIT_MAX_WIDTH];
}

void ScrollEditViewEvent(EDITOR_VIEW* theView, UINT32 eventCode, bool repeating)
// send a scroll event to the view
{
	VIEW_EVENT theRecord;
	EDITOR_KEY theKeyData;

	if (!repeating)
	{
		theRecord.eventType = VET_KEYDOWN;
	}
	else
	{
		theRecord.eventType = VET_KEYREPEAT;
	}
	theRecord.view = theView;
	theKeyData.isVirtual = true;
	theKeyData.keyCode = eventCode;		// this is the key
	theKeyData.modifiers = 0;
	theRecord.eventData = &theKeyData;
	HandleViewEvent(&theRecord);
}

void ScrollEditViewThumbVerticalEvent(EDITOR_VIEW* theView, UINT32 newValue)
// tell the editor that the view in the window with theScrollBar is to be positioned vertically
{
	VIEW_EVENT theRecord;
	VIEW_POS_EVENT_DATA thePosData;

	theRecord.eventType = VET_POSITIONVERTICAL;
	theRecord.view = theView;
	thePosData.modifiers = 0;
	thePosData.position = newValue;
	theRecord.eventData = &thePosData;
	HandleViewEvent(&theRecord);
}

void ScrollEditViewThumbHorizontalEvent(EDITOR_VIEW* theView, UINT32 newValue)
// tell the editor that the view in the window with theScrollBar is to be positioned horizontally
{
	VIEW_EVENT theRecord;
	VIEW_POS_EVENT_DATA thePosData;

	theRecord.eventType = VET_POSITIONHORIZONTAL;
	theRecord.view = theView;
	thePosData.modifiers = 0;
	thePosData.position = newValue;
	theRecord.eventData = &thePosData;
	HandleViewEvent(&theRecord);
}

void UpdateEditorWindows()
// redraw all editor windows that have been invalidated by previous expose or
// graphics expose events. This goes from the top window to the bottom one.
{
	WINDOWLISTELEMENT* curWindowElement;

	curWindowElement = g_windowListHead;
	while (curWindowElement)
	{
		[curWindowElement->windowRef displayIfNeeded];
		curWindowElement = curWindowElement->nextElement;
	}
}

void MinimizeDocumentWindow(EDITOR_WINDOW* theWindow)
// minimize the passed window
{
	WINDOWLISTELEMENT* theWindowElement;

	theWindowElement = (WINDOWLISTELEMENT*)theWindow->userData;
	[theWindowElement->windowRef miniaturize:nil];
}

void UnminimizeDocumentWindow(EDITOR_WINDOW* theWindow)
// unminimize the passed window, this really should be removed in favor of RestoreDocumentWindow
{
	WINDOWLISTELEMENT* theWindowElement;

	theWindowElement = (WINDOWLISTELEMENT*)theWindow->userData;
	[theWindowElement->windowRef deminiaturize:nil];
}

void MaximizeDocumentWindow(EDITOR_WINDOW* theWindow)
// minimize the passed window
{
	WINDOWLISTELEMENT* theWindowElement;

	theWindowElement = (WINDOWLISTELEMENT*)theWindow->userData;
	if (!theWindowElement->windowRef.zoomed)							// if we are not zoomed already
	{
		[theWindowElement->windowRef zoom:nil];							// then zoom
	}
}

void RestoreDocumentWindow(EDITOR_WINDOW* theWindow)
// minimize the passed window
{
	WINDOWLISTELEMENT* theWindowElement;

	theWindowElement = (WINDOWLISTELEMENT*)theWindow->userData;
	if (theWindowElement->windowRef.zoomed)								// if we are zoomed
	{
		[theWindowElement->windowRef zoom:nil];							// unzoom
	}
}

void SetTopDocumentWindow(EDITOR_WINDOW* theWindow)
// Set the passed document window to the top of the display
// Don't move it to the top of the WINDOWLISTELEMENT list here, wait for the activate event
{
	WINDOWLISTELEMENT* theWindowElement;

	theWindowElement = (WINDOWLISTELEMENT*)theWindow->userData;
	[theWindowElement->windowRef makeKeyAndOrderFront:nil];
}

EDITOR_WINDOW* GetActiveDocumentWindow()
// return the document window that has the user's attention, or NULL if there is none
// depending on the system, this may or may not return the top-most document window
{
	WINDOWLISTELEMENT* theWindowElement;

	if ((theWindowElement = LocateActiveEditorWindow()))		// should be the top of the list
	{
		return (theWindowElement->theEditorWindow);
	}
	return (NULL);
}

EDITOR_WINDOW* GetTopDocumentWindow()
// return the top-most document window, or NULL if there is none
{
	if (g_windowListHead)
	{
		return (g_windowListHead->theEditorWindow);
	}
	return (NULL);
}

EDITOR_WINDOW* GetNextDocumentWindow(EDITOR_WINDOW* previousWindow)
// return the document just under previousWindow, or NULL if there is none
{
	WINDOWLISTELEMENT* theWindowElement;

	theWindowElement = (WINDOWLISTELEMENT*)previousWindow->userData;
	if (theWindowElement->nextElement)
	{
		return ((EDITOR_WINDOW*)theWindowElement->nextElement->theEditorWindow);
	}
	return (NULL);
}

EDITOR_VIEW* GetDocumentWindowCurrentView(EDITOR_WINDOW* theWindow)
// passed a document window, return the current view of the window
{
	return ((EDITOR_VIEW*)theWindow->windowInfo);
}

bool GetDocumentWindowTitle(EDITOR_WINDOW* theWindow, char* theTitle, UINT32 stringBytes)
// copy theWindow's title into theTitle
// if theWindow's title is longer than stringBytes, return FALSE
{
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theViewInfo = theView->viewInfo;
	if ((strlen(theViewInfo->titleString) + 1) <= stringBytes)
	{
		strcpy(theTitle, theViewInfo->titleString);
		return (true);
	}
	return (false);
}

bool SetDocumentWindowTitle(EDITOR_WINDOW* theWindow, const char* theTitle)
// replace theTitle of theWindow, if there is a problem, return FALSE
{
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;
	char* newTitle;
	WINDOWLISTELEMENT* theWindowElement;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theViewInfo = theView->viewInfo;
	theWindowElement = (WINDOWLISTELEMENT*)theWindow->userData;
	if ((newTitle = (char*)MNewPtr((UINT32)(strlen(theTitle) + 1))))
	{
		strcpy(newTitle, theTitle);
		MDisposePtr((void*)theViewInfo->titleString);
		theViewInfo->titleString = newTitle;
		theWindowElement->windowRef.title = [NSString stringWithCString:theViewInfo->titleString encoding:NSASCIIStringEncoding];
		return (true);
	}
	return (false);
}

bool GetSortedDocumentWindowList(UINT32* numElements, EDITOR_WINDOW** *theList)
// return a list of all document windows -- sorted top to bottom
// if there is a problem, return FALSE
// otherwise return TRUE. If TRUE is returned, theList must be
// MDisposePtr'ed by the caller (even if numElements is 0)
{
	WINDOWLISTELEMENT* theElement;
	unsigned int child;
	unsigned int numChildren;
	bool fail;

	fail = false;
	*numElements = 0;																	// no elements yet

	theElement = g_windowListHead;

	numChildren = 0;
	while (theElement)
	{
		theElement = theElement->nextElement;
		numChildren++;
	}
	if (((*theList) = (EDITOR_WINDOW**)MNewPtr(sizeof(EDITOR_WINDOW*)*numChildren)))		// create a list of pointers large enough to hold the maximum number of windows located
	{
		theElement = g_windowListHead;
		for (child = numChildren; child > 0; child--)
		{
			if (theElement->theEditorWindow->windowType == EWT_DOCUMENT)				// see if it belongs on the list
			{
				(*theList)[(*numElements)++] = theElement->theEditorWindow;			// assign it to the list
			}
			theElement = theElement->nextElement;
		}
	}
	else
	{
		fail = true;
	}
	return (!fail);
}

bool GetEditorDocumentWindowForegroundColor(EDITOR_WINDOW* theWindow, EDITOR_COLOR* foregroundColor)
// get the foreground color in use by theWindow
// if there is a problem, SetError, and return false
{
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theViewInfo = theView->viewInfo;
	(*foregroundColor) = theViewInfo->foregroundColor;

	return (true);
}

bool SetEditorDocumentWindowForegroundColor(EDITOR_WINDOW* theWindow, EDITOR_COLOR foregroundColor)
// set the foreground color to be used in theWindow
// if there is a problem, leave the color unchanged, set the error
// and return false
{
	EDITOR_VIEW* theView;
	WINDOWLISTELEMENT* theWindowElement;

	if (SetDocumentForegroundColor(theWindow, foregroundColor))
	{
		theView = (EDITOR_VIEW*)theWindow->windowInfo;
		theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;
		theWindowElement->viewRef.needsDisplay = YES;
		return (true);
	}
	return (false);
}

bool GetEditorDocumentWindowBackgroundColor(EDITOR_WINDOW* theWindow, EDITOR_COLOR* backgroundColor)
// get the background color in use by theWindow
// if there is a problem, SetError, and return false
{
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theViewInfo = theView->viewInfo;
	(*backgroundColor) = theViewInfo->backgroundColor;

	return (true);
}

bool SetEditorDocumentWindowBackgroundColor(EDITOR_WINDOW* theWindow, EDITOR_COLOR backgroundColor)
// set the background color to be used in theWindow
// if there is a problem, leave the color unchanged, set the error
// and return false
{
	EDITOR_VIEW* theView;
	WINDOWLISTELEMENT* theWindowElement;

	if (SetDocumentBackgroundColor(theWindow, backgroundColor))
	{
		theView = (EDITOR_VIEW*)theWindow->windowInfo;
		theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;
		theWindowElement->viewRef.needsDisplay = YES;
		return (true);
	}
	return (false);
}

bool GetEditorDocumentWindowRect(EDITOR_WINDOW* theWindow, EDITOR_RECT* theRect)
// get the rectangle of theWindow in global space
{
	EDITOR_VIEW* theView;
	WINDOWLISTELEMENT* theWindowElement;
	NSRect wRect;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;

	wRect = [theWindowElement->windowRef frame];
	theRect->x = wRect.origin.x;
	theRect->y = wRect.origin.y;
	theRect->w = wRect.size.width;
	theRect->h = wRect.size.height;
	return (true);
}

bool SetEditorDocumentWindowRect(EDITOR_WINDOW* theWindow, EDITOR_RECT* theRect)
// set theWindow to theRect
{
	EDITOR_VIEW* theView;
	WINDOWLISTELEMENT* theWindowElement;
	NSRect wRect;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;

	wRect.origin.x = theRect->x;
	wRect.origin.y = theRect->y;
	wRect.size.width = theRect->w;
	wRect.size.height = theRect->h;
	[theWindowElement->windowRef setFrame:wRect display:YES];
	return (true);
}

void GetEditorScreenDimensions(UINT32* theWidth, UINT32* theHeight)
// Return the size of the screen of the main monitor
{
	NSRect screenRect;
	NSArray* screenArray = [NSScreen screens];
	NSUInteger screenCount = [screenArray count];
	NSUInteger index;

	for (index = 0; index < screenCount; index++)
	{
		NSScreen* screen = [screenArray objectAtIndex:index];
		screenRect = [screen visibleFrame];
	}
	*theWidth = screenRect.size.width;
	*theHeight = screenRect.size.height;
}

void GetEditorScreenOffsetAndDimensions(INT32* xOffset, INT32* yOffset, UINT32* width, UINT32* height)
// Return the size of the screen of the main monitor
{
	*xOffset = 0;
	*yOffset = 0;
	NSRect screenRect;
	NSArray* screenArray = [NSScreen screens];
	NSUInteger screenCount = [screenArray count];
	NSUInteger index;

	for (index = 0; index < screenCount; index++)
	{
		NSScreen* screen = [screenArray objectAtIndex:index];
		screenRect = [screen visibleFrame];
	}
	*width = screenRect.size.width;
	*height = screenRect.size.height;
}

static bool GetNewWindowsViews(WINDOWLISTELEMENT* theWindowElement)
// Get references to all the new windows HIViews and return true, else false if there was an issue
{
	NSRect winRect = [theWindowElement->windowRef.contentView frame];

	CGFloat sWidth = [NSScroller scrollerWidthForControlSize:NSControlSizeRegular scrollerStyle:[NSScroller preferredScrollerStyle]];

	theWindowElement->viewRef = [[GUIView alloc] initWithFrame:NSMakeRect(0, (kStatusBarHeight + kToolBarHeight), winRect.size.width - sWidth, winRect.size.height - sWidth - (kStatusBarHeight + kToolBarHeight))];
	theWindowElement->viewRef.autoresizingMask = NSViewHeightSizable | NSViewWidthSizable;
	[[theWindowElement->windowRef contentView] addSubview:theWindowElement->viewRef];
	theWindowElement->viewRef.windowElement = theWindowElement;

	theWindowElement->statusRef = [[GUIStatusView alloc] initWithFrame:NSMakeRect(0, 0, winRect.size.width - sWidth, kStatusBarHeight)];
	theWindowElement->statusRef.autoresizingMask = NSViewWidthSizable;
	theWindowElement->statusRef.windowElement = theWindowElement;
	[[theWindowElement->windowRef contentView] addSubview:theWindowElement->statusRef];

	theWindowElement->toolBarRef = [[GUIToolView alloc] initWithFrame:NSMakeRect(0, kStatusBarHeight, winRect.size.width - sWidth, kToolBarHeight)];
	theWindowElement->toolBarRef.autoresizingMask = NSViewWidthSizable;
	theWindowElement->toolBarRef.windowElement = theWindowElement;
	[[theWindowElement->windowRef contentView] addSubview:theWindowElement->toolBarRef];

	theWindowElement->vScrollBarViewRef = [[NSScroller alloc] initWithFrame:NSMakeRect(winRect.size.width - sWidth, 0, sWidth, winRect.size.height - sWidth)];
	[theWindowElement->vScrollBarViewRef setEnabled:YES];
	theWindowElement->vScrollBarViewRef.arrowsPosition = NSScrollerArrowsDefaultSetting;
	theWindowElement->vScrollBarViewRef.scrollerStyle = [NSScroller preferredScrollerStyle];
	theWindowElement->vScrollBarViewRef.controlSize = NSControlSizeRegular;
	theWindowElement->vScrollBarViewRef.autoresizingMask = NSViewHeightSizable | NSViewMinYMargin | NSViewMinXMargin;
	[[theWindowElement->windowRef contentView] addSubview:theWindowElement->vScrollBarViewRef];
	[theWindowElement->vScrollBarViewRef setTarget:windowDelegateHandler];
	[theWindowElement->vScrollBarViewRef setAction:@selector(scrollerAction:)];

	theWindowElement->hScrollBarViewRef = [[NSScroller alloc] initWithFrame:NSMakeRect(0, winRect.size.height - sWidth, winRect.size.width - sWidth, sWidth)];
	[theWindowElement->hScrollBarViewRef setEnabled:YES];
	theWindowElement->hScrollBarViewRef.arrowsPosition = NSScrollerArrowsDefaultSetting;
	theWindowElement->hScrollBarViewRef.scrollerStyle = [NSScroller preferredScrollerStyle];
	theWindowElement->hScrollBarViewRef.controlSize = NSControlSizeRegular;
	theWindowElement->hScrollBarViewRef.autoresizingMask = NSViewWidthSizable | NSViewMinXMargin | NSViewMinYMargin;
	[[theWindowElement->windowRef contentView] addSubview:theWindowElement->hScrollBarViewRef];
	[theWindowElement->hScrollBarViewRef setTarget:windowDelegateHandler];
	[theWindowElement->hScrollBarViewRef setAction:@selector(scrollerAction:)];

	return (true);
}

EDITOR_WINDOW* OpenDocumentWindow(EDITOR_BUFFER* theBuffer, EDITOR_RECT* theRect, char* theTitle, char* fontName, UINT32 tabSize, EDITOR_COLOR foreground, EDITOR_COLOR background)
// open a document window onto theBuffer, place it at theRect give the new window theTitle
// if there is a problem, undo anything that was done, and return NULL
{
	EDITOR_WINDOW* theWindow;
	char* title;
	EDITOR_VIEW_DESCRIPTOR viewDescriptor;
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;
	WINDOWLISTELEMENT* theWindowElement;

	if (windowDelegateHandler == nil)
	{
		windowDelegateHandler = [[WindowDelegateHandlerObject alloc] init];
	}

	if ((theWindow = (EDITOR_WINDOW*)MNewPtrClr((UINT32)sizeof(EDITOR_WINDOW))))								// create editor window
	{
		if ((title = (char*)MNewPtrClr((UINT32)(strlen(theTitle) + 1))))
		{
			if ((theWindowElement = new WINDOWLISTELEMENT()))
			{
				theWindow->buffer = theBuffer;
				theWindowElement->theEditorWindow = theWindow;													// point to our high level concept of window
				theWindow->userData = theWindowElement;															// point back to the window element
				LinkWindowElement(theWindowElement);
				strcpy(title, theTitle);

				theWindow->windowType = EWT_DOCUMENT;															// set the editor window type to "document"

				viewDescriptor.buffer = theBuffer;
				viewDescriptor.rect.x = theRect->x;																// place where the view is going
				viewDescriptor.rect.w = theRect->w;
				viewDescriptor.rect.y = theRect->y;
				viewDescriptor.rect.h = theRect->h;
				viewDescriptor.topLine = 0;
				viewDescriptor.leftPixel = 0;
				viewDescriptor.tabSize = tabSize;
				viewDescriptor.fontName = fontName;
				viewDescriptor.foregroundColor = foreground;
				viewDescriptor.backgroundColor = background;
				viewDescriptor.active = true;																	// create the view active
				viewDescriptor.viewTextChangedVector = AdjustDocumentWindowScrollThumbsAndStatus;				// fix the thumbs, and status when the view text changes
				viewDescriptor.viewPositionChangedVector = AdjustDocumentWindowScrollThumbs;					// fix the thumbs when the view position changes
				viewDescriptor.viewSelectionChangedVector = AdjustDocumentWindowViewStatus;						// fix the status when the selection changes
				if ((theWindow->windowInfo = CreateEditorView(theWindow, &viewDescriptor)))
				{
					if (SetDocumentBackgroundColor(theWindow, background) && SetDocumentForegroundColor(theWindow, foreground))
					{
						NSRect frame = NSMakeRect(theRect->x, theRect->y, theRect->w, theRect->h);
						theWindowElement->windowRef = [[NSWindow alloc] initWithContentRect:frame
													   styleMask:NSWindowStyleMaskTitled | NSWindowStyleMaskMiniaturizable | NSWindowStyleMaskResizable | NSWindowStyleMaskClosable
													   backing:NSBackingStoreBuffered
													   defer:NO];
						if (theWindowElement->windowRef)
						{
							[theWindowElement->windowRef setReleasedWhenClosed:NO];
							theWindowElement->windowRef.showsResizeIndicator = YES;
							theWindowElement->windowRef.title = [NSString stringWithCString:theTitle encoding:NSASCIIStringEncoding];
							theWindowElement->windowRef.contentView = [[ContentView alloc] initWithFrame:frame];
							theWindowElement->windowRef.delegate = windowDelegateHandler;

							theView = (EDITOR_VIEW*)theWindow->windowInfo;
							theView->parentWindow->userData = (void*)theWindowElement;
							theViewInfo = theView->viewInfo;
							theViewInfo->titleString = title;
							//
							// We have multiple NSViews in the window, main text view, status below the title bar,
							// scrollbars, and tool bar. Create all the views.
							//
							if (GetNewWindowsViews(theWindowElement))
							{
								//
								//	Create a tracking area in the text view and respond to kEventControlTrackingAreaEntered and kEventControlTrackingAreaExited events
								//	to change the cursor from IBeam to arrow when entering or exiting the view. This is much easier than looking at mouse move, mouse leave and mouse enter
								//	events for the window. Besides we don't seem to always get mouse leave events?
								//
								[theWindowElement->windowRef makeKeyAndOrderFront:NSApp];
								theWindowElement->windowInitialized = false;
								return (theWindow);
							}
							theWindowElement->windowRef = nil;
						}
					}
					DisposeEditorView((EDITOR_VIEW*)theWindow->windowInfo);
				}
				UnlinkWindowElement(theWindowElement);
				delete theWindowElement;
			}
			MDisposePtr(title);
		}
		MDisposePtr(theWindow);
	}
	return ((EDITOR_WINDOW*)NULL);
}

void CloseDocumentWindow(EDITOR_WINDOW* theWindow)
// close an open document window
{
	GUI_VIEW_INFO* theViewInfo;
	EDITOR_VIEW* theView;
	WINDOWLISTELEMENT* theWindowElement;

	theView = (EDITOR_VIEW*)theWindow->windowInfo;
	theViewInfo = theView->viewInfo;
	theWindowElement = (WINDOWLISTELEMENT*)theWindow->userData;
	[theWindowElement->viewRef removeFromSuperviewWithoutNeedingDisplay];
	[theWindowElement->statusRef removeFromSuperviewWithoutNeedingDisplay];
	[theWindowElement->toolBarRef removeFromSuperviewWithoutNeedingDisplay];
	[theWindowElement->vScrollBarViewRef removeFromSuperviewWithoutNeedingDisplay];
	[theWindowElement->hScrollBarViewRef removeFromSuperviewWithoutNeedingDisplay];

	UnlinkWindowElement(theWindowElement);
	MDisposePtr(theViewInfo->titleString);
	DisposeEditorView(theView);
	theWindowElement->windowRef.delegate = nil;
	[theWindowElement->windowRef close];
	theWindowElement->windowRef = nil;
	delete theWindowElement;
	MDisposePtr(theWindow);

	EDITOR_WINDOW* topWindow = GetTopDocumentWindow();
	if (topWindow)
	{
		SetTopDocumentWindow(topWindow);
	}

}




