// Low level stuff needed to init machine
// Copyright (C) 2009 Fred Allen.
// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

#include	"includes.h"

void UnInitEnvironment()
// undo what Init did
{
}

bool InitEnvironment()
// Initialize the machine, software libraries, etc...
// if there is a problem, set the error and return false
{
	return(true);
}

void EarlyUnInit()
// This is the very last thing that the editor calls before exiting
{
	UninitToolBar();
	UninitStatusBar();
	UnInitEvents();
	UninitMenuManager();
}

static bool SetCurrentDirectoryToExecutable()
// set the current directory to that of the executable
{
	NSString* unixPath = [[NSBundle mainBundle] resourcePath];
	const char* programPath = [unixPath cStringUsingEncoding:NSUTF8StringEncoding];

	chdir(programPath);
	return(true);
}

static void InitGlobals()
{
	g_currentPasteboard = NULL;
}

bool EarlyInit()
// This is called before anything else in the editor, it can be used to fork off the
// editor as a separate process, or do anything which may need to be done very early
// if this returns false, the editor will exit without complaint
{
	InitGlobals();
	if (SetCurrentDirectoryToExecutable())
	{
		if (InitMenuManager())
		{
			if (InitEvents())
			{
				if (InitStatusBar())
				{
					if (InitToolBar())
					{
						return(true);
					}
				}
				UnInitEvents();
			}
			UninitMenuManager();
		}
	}
	return(false);
}

