// Copyright (C) 2009 Fred Allen.
//
// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".


#include "includes.h"

//@@@@@@@@@@@@@@@@@@@@@@@@@@@
// still to do
//@@@@@@@@@@@@@@@@@@@@@@@@@@@
static PMPageFormat CreateDefaultPageFormatObject()
{
	PMPageFormat pageFormat;

	if (PMNewPageFormat(&pageFormat) == noErr)
	{
		PMDefaultPageFormat(pageFormat);
	}
	return (pageFormat);
}

void PrintView(EDITOR_VIEW* view)
{
	GUI_VIEW_INFO* theViewInfo;
	WINDOWLISTELEMENT* theWindowElement;
	PMPrintSettings printSettings;
	PMPrintSession printSession;
	UInt32 minPage;
	UInt32 maxPage;
	PMPageFormat pageFormat;
	UINT32 startLine;
	UINT32 numLines;
	UINT32 width;
	UINT32 startPosition;						// selection information
	UINT32 endPosition;
	UINT32 endLine;
	UINT32 startLinePosition;
	UINT32 endLinePosition;
	UINT32 totalSegments;
	UINT32 totalSpan;

	theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;
	theViewInfo = theView->viewInfo;															// point at the information record for this view
	EditorGetSelectionInfo(theView->parentBuffer, theView->parentBuffer->selectionUniverse, &startPosition, &endPosition, &startLine, &endLine, &startLinePosition, &endLinePosition, &totalSegments, &totalSpan);

	if (PMCreateSession(&printSession) == noErr)
	{
		if (PMNewPageFormat(&pageFormat) == noErr)
		{
			PMDefaultPageFormat(pageFormat);

			if (PMSessionValidatePageFormat(printSession, pageFormat, kPMDontWantBoolean) == noErr)
			{
				if (PMCreatePrintSettings(&printSettings) == noErr)
				{
					if (PMSessionDefaultPrintSettings(printSession, printSettings) == noErr)
					{
						CFStringRef titleRef;

						titleRef = CFStringCreateWithCString(kCFAllocatorDefault, theViewInfo->titleString, kCFStringEncodingMacRoman);
						PMSetJobNameCFString(printSettings, titleRef);
						CFRelease(titleRef);
					}
					PMRelease(printSession);
				}
			}
		}
	}


	OSStatus DoPrint(WindowRef parentWindow, void* documentDataP, Boolean printOne)
	{
		OSStatus err = noErr;
		PMPrintSettings printSettings = NULL;

		UInt32 minPage = 1, maxPage;
		PMPrintSession printSession;
		err = PMCreateSession(&printSession);
		if (err == noErr)
		{
			// validate the page format we're going to use
			err = PMSessionValidatePageFormat(printSession,
											  GetPageFormatFromPrivateData(documentDataP),
											  kPMDontWantBoolean);
			if (err == noErr)
			{
				err = PMCreatePrintSettings(&printSettings);
				if (err == noErr)
				{
					err = PMSessionDefaultPrintSettings(printSession, printSettings);
					if (err == noErr)
					{
						CFStringRef myDocumentNameRef;
						err = CopyDocumentName(documentDataP, &myDocumentNameRef);
						if (err == noErr)
						{
							// set the job name before displaying the print dialog
							err = PMSetJobNameCFString (printSettings, myDocumentNameRef);
							CFRelease(myDocumentNameRef);	// release our reference to the document name
						}
					}
				}
				if (err == noErr)
				{
					/*
					    On Mac OS X, calling PMSetPageRange has the following benefits:
					    (a) sets the From/To settings in the print dialog to the range of pages
					        in the document

					        AND

					    (b) the print dialog code enforces this so that users can't enter
					        values outside this range.
					*/
					maxPage = MyGetDocumentNumPagesInDoc(documentDataP);
					err = PMSetPageRange(printSettings, minPage, maxPage);
				}

				// register our custom PDE if we haven't already
				if (err == noErr)
				{
					static Boolean initedPDE = false;
					if (!initedPDE)
					{
						err = MyRegisterPDE(kMyPDEName);
						initedPDE = true;
					}
				}

				if (err == noErr)
				{
					Boolean accepted;
					static PMSheetDoneUPP myPrintDialogDoneProc = NULL;
					if (!myPrintDialogDoneProc)
					{
						myPrintDialogDoneProc = NewPMSheetDoneUPP(MyPrintDialogDoneProc);
						if (!myPrintDialogDoneProc)
						{
							err = memFullErr;
						}
					}

					if (!err)
					{
						Boolean sheetsAreAvailable = TRUE;
						err = SetPrintSettingsOnPrivateData(documentDataP, printSettings);
						if (!err)
						{
							err = PMSessionUseSheets(printSession, parentWindow, myPrintDialogDoneProc);
							if (err == kPMNotImplemented)
							{
								// we get here if sheets are not available, i.e. Mac OS 8/9
								err = noErr;
								sheetsAreAvailable = FALSE;
							}
							if (err == noErr && !printOne)
							{
								err = PMSessionPrintDialog(printSession, printSettings,
														   GetPageFormatFromPrivateData(documentDataP),
														   &accepted);
								/*  when using sheets, the value of 'accepted' returned here is irrelevant
								    since it is our sheet done proc that is called when the dialog is dismissed.
								    Our dialog done proc will be called when the sheet is dismissed.

								    If sheets are NOT implemented then WE call our DialogDone proc here
								    to complete the dialog.
								*/
								if (err == noErr && !sheetsAreAvailable)
								{
									MyPrintDialogDoneProc(printSession, parentWindow, accepted);
								}
							}
							else
							{
								// if we are doing print one we have no dialog, therefore
								// we have to call our dialog done proc since there is no
								// dialog to do so for us
								if (err == noErr)
								{
									MyPrintDialogDoneProc(printSession, parentWindow, TRUE);
								}
							}
						}
					}
				}
			}
			if (err != noErr)
			{
				/* normally the printSettings and printSession would be released by our dialog
				done proc but if we got an error and therefore got to this point in our code,
					the dialog done proc was NOT called and therefore we need to release the
					printSession here
				          */
				if (printSettings)
				{
					// this releases any print settings already stored on our private data
					(void)SetPrintSettingsOnPrivateData(documentDataP, NULL);
				}
				(void)PMRelease(printSession);   // ignoring error since we already have one
			}
		}

		DoErrorAlert(err, kMyPrintErrorFormatStrKey);
		return err;
	}

// -------------------------------------------------------------------------------------