// Shell commands specific to OS X version of e93
// Copyright (C) 2009 Fred Allen.
//
// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

#include "includes.h"
#import <AppKit/AppKit.h>

// routines callable by the editor

#if 0 // unused???
static bool GetBoolean(Tcl_Interp* localInterpreter, const char* theString, bool* theValue)
// convert theString to a boolean value
// return FALSE if the conversion failed, and fill the Tcl result
{
	int intValue;

	if (Tcl_GetBoolean(localInterpreter, theString, &intValue) == TCL_OK)
	{
		*theValue = intValue ? true : false;
		return (true);
	}
	return (false);
}
#endif

static bool GetUINT32(Tcl_Interp* localInterpreter, const char* theString, UINT32* theNumber)
// convert theString to a UINT32
// return FALSE if the conversion failed, and fill the Tcl result
{
	int theInt;

	if (Tcl_GetInt(localInterpreter, theString, &theInt) == TCL_OK)
	{
		*theNumber = (UINT32)theInt;					// ### this should really be fixed to be a true unsigned
		return (true);
	}
	return (false);
}

static int Cmd_Print(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
// attempt to Maximize the given document window
{
	if (argc == 2)
	{
		EDITOR_WINDOW* window;
		
		if ((window = LocateEditorDocumentWindow((char*)argv[1])))
		{
			EDITOR_VIEW* theView;
			WINDOWLISTELEMENT* theWindowElement;

			theView = (EDITOR_VIEW*)window->windowInfo;
			theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;
			
			[[NSPrintOperation printOperationWithView:theWindowElement->viewRef] runOperation];
			return (TCL_OK);
		}
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " windowName\"", NULL);
	}
	return (TCL_ERROR);
}

static int Cmd_DebugPrint(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
// attempt to Maximize the given document window
{
	if (argc == 2)
	{
		printf("%s\n", (char*)argv[1]);
		return (TCL_OK);
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " windowName\"", NULL);
	}
	return (TCL_ERROR);
}

static int Cmd_MaximizeWindow(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
// attempt to Maximize the given document window
{
	EDITOR_WINDOW* window;

	if (argc == 2)
	{
		if ((window = LocateEditorDocumentWindow((char*)argv[1])))
		{
			MaximizeDocumentWindow(window);
			return (TCL_OK);
		}
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " windowName\"", NULL);
	}
	return (TCL_ERROR);
}

static int Cmd_RestoreWindow(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
// attempt to Restore the given document window to its normal state
{
	EDITOR_WINDOW* window;

	if (argc == 2)
	{
		if ((window = LocateEditorDocumentWindow((char*)argv[1])))
		{
			RestoreDocumentWindow(window);
			return (TCL_OK);
		}
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " windowName\"", NULL);
	}
	return (TCL_ERROR);
}

static int Cmd_HideApplication(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
// attempt to Restore the given document window to its normal state
{
	return (TCL_OK);
}

static int Tcl_SetGlobalBottomBarText(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
//	Passed a positionNumber, a width and a string, display the string in the bottom bar
//	in a box that is width pixels wide, clipping the string to fit, at a relative position
//	from left to right base off fieldNumber. If width is zero, any status entry that was displayed
//	is removed.
{
	UINT32 position;
	UINT32 width;

	if (argc == 4)
	{
		if (GetUINT32(localInterpreter, argv[1], &position))
		{
			if (GetUINT32(localInterpreter, argv[2], &width))
			{
				if (SetGlobalBottomBarText(position, width, argv[3]))
				{
					return (TCL_OK);
				}
				else
				{
					Tcl_AppendResult(localInterpreter, GetErrorTrace(), NULL);
				}
			}
		}
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " fieldNumber width string\"", NULL);
	}
	return (TCL_ERROR);
}

static int Tcl_SetGlobalBottomBarButton(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
//	Passed a fieldNumber, a width and a string, display the string in the status bar
//	in a box that is width pixels wide, clipping the string to fit, at a relative position
//	from left to right base off fieldNumber. If width is zero, any status entry that was displayed
//	is removed.
{
	UINT32 position;
	UINT32 width;

	if (argc == 5)
	{
		if (GetUINT32(localInterpreter, argv[1], &position))
		{
			if (GetUINT32(localInterpreter, argv[2], &width))
			{
				if (SetGlobalBottomBarButton(position, width, argv[3], argv[4]))
				{
					return (TCL_OK);
				}
				else
				{
					Tcl_AppendResult(localInterpreter, GetErrorTrace(), NULL);
				}
			}
		}
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " fieldNumber width string cmdString\"", NULL);
	}
	return (TCL_ERROR);
}

static int Tcl_GetBottomBarStringWidth(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
//	Passed a string, return the number of pixels wide the string is for the current font selected in the global
//	status bar.
{
	UINT32 width;
	char tempString[256];

	if (argc == 2)
	{
		width = GetBottomBarStringWidth(argv[1]);
		sprintf(tempString, "%u", width);
		Tcl_AppendResult(localInterpreter, tempString, NULL);
		return (TCL_OK);
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " string\"", NULL);
	}
	return (TCL_ERROR);
}

static int Tcl_BringupPopupMenu(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
// Pass in a list in argv, bring up a popup menu and return whether the user click on an item or not and which item
{
	int result;
	int pathArgc;
	const char** pathArgv;
	UINT32 item;
	bool itemSelected;

	result = TCL_ERROR;
	if (argc >= 2)
	{
		if (Tcl_SplitList(localInterpreter, argv[1], &pathArgc, &pathArgv) == TCL_OK)
		{
			if (CreatePopupMenuAndReturnItem(pathArgc, (char**)pathArgv, &item, &itemSelected))
			{
				if (itemSelected)
				{
					Tcl_AppendElement(localInterpreter, pathArgv[item]);
				}
				result = TCL_OK;
				ClearAbort();				// if user managed to send break sequence during the popup menu, we would rather not abort now!
			}
			else
			{
				Tcl_AppendResult(localInterpreter, GetErrorTrace(), NULL);
			}
			ckfree((char*)pathArgv);
		}
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " list\"", NULL);
	}
	return (result);
}

static int Tcl_SpellCheck(ClientData clientData, Tcl_Interp* localInterpreter, int argc, const char* argv[])
{
	int result;

	result = TCL_ERROR;
	if (argc >= 2)
	{
		NSString *word = [NSString stringWithUTF8String:argv[1]];

		NSSpellChecker* checker = [NSSpellChecker sharedSpellChecker]; 
		NSRange range = NSMakeRange(0, [word length]); 


		NSArray *checkingResult = [checker guessesForWordRange:range
		                                    inString:word
		                                    language:@"en"
		                      inSpellDocumentWithTag:0];

		if ([checkingResult count] == 0)
		{
		}
		else
		{
			for (NSString* str in checkingResult)
			{
				const char *cStr = [str UTF8String];
				Tcl_AppendElement(localInterpreter, cStr);
			}
			result = TCL_OK;
		}
	}
	else
	{
		Tcl_AppendResult(localInterpreter, "wrong # args: should be \"", argv[0], " list\"", NULL);
	}
	return (result);
}


bool AddSupplementalShellCommands(Tcl_Interp* interpreter)
// if a given environment desires additional shell commands
// it can add them here
// if there is a problem, SetError, and return false
{
	Tcl_CreateCommand(interpreter, "macprint", Cmd_Print, NULL, NULL);
	Tcl_CreateCommand(interpreter, "debugprint", Cmd_DebugPrint, NULL, NULL);
	Tcl_CreateCommand(interpreter, "maximizewindow", Cmd_MaximizeWindow, NULL, NULL);
	Tcl_CreateCommand(interpreter, "restorewindow", Cmd_RestoreWindow, NULL, NULL);
	Tcl_CreateCommand(interpreter, "hideapplication", Cmd_HideApplication, NULL, NULL);
	Tcl_CreateCommand(interpreter, "getbottombarstringwidth", Tcl_GetBottomBarStringWidth, NULL, NULL);
	Tcl_CreateCommand(interpreter, "setglobalbottombartext", Tcl_SetGlobalBottomBarText, NULL, NULL);
	Tcl_CreateCommand(interpreter, "setglobalbottombarbutton", Tcl_SetGlobalBottomBarButton, NULL, NULL);
	Tcl_CreateCommand(interpreter, "bringuppopupmenu", Tcl_BringupPopupMenu, NULL, NULL);
	Tcl_CreateCommand(interpreter, "spellcheck", Tcl_SpellCheck, NULL, NULL);
	return (true);
}

bool ExecuteTclCommand(const char* command)
{
	int tclResult;

	ClearAbort();
	tclResult = Tcl_Eval(tclInterpreter, command);
	if ((tclResult != TCL_OK) && *(tclInterpreter->result))
	{
		ReportMessage("Tcl execute error:\n%.256s\n", tclInterpreter->result);
	}
	return (true);
}

