// Copyright (C) 2009 Fred Allen.
//
// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".


#include "includes.h"

#define	RUNNING_TASK_STR			"Running"
#define	IDLE_TASK_STR				"Idle"

#define	STATUS_TEXT_Y_OFFSET		1
#define	STATUS_TASK_TEXT_X_OFFSET	5
#define	STATUS_DIRTY_TEXT_X_OFFSET	90
#define	STATUS_POS_TEXT_X_OFFSET	111

#define	STATUS_BACK_COLOR_R			0.93
#define	STATUS_BACK_COLOR_G			0.93
#define	STATUS_BACK_COLOR_B			0.93

#define	STATUS_LINE_COLOR_R			0.5
#define	STATUS_LINE_COLOR_G			0.5
#define	STATUS_LINE_COLOR_B			0.5

#define	STATUS_TEXT_COLOR_R			0.0
#define	STATUS_TEXT_COLOR_G			0.0
#define	STATUS_TEXT_COLOR_B			0.0

#define	STATUS_BAR_FONT_NAME		"-*-Lucida Grande-normal-r-normal--15--75-75-*-70-*-*"

static EDITOR_FONT* statusBarFont;

//static char localErrorFamily[] = "OSXgui";

enum
{
	STATUS_POSITION_OUT_OF_RANGE
};

#if 0 // unused???
static const char* errorMembers[] =
{
	"StatusPositionOutOfRange"
};

static const char* errorDescriptions[] =
{
	"Field number for SetBottomBarPosition is out of range"
};
#endif

void DrawWindowStatusBar(EDITOR_VIEW* theView, CGContextRef cgContext)
// Draw the editor status bar info
{
	char statusStr[512];
	UINT32 startPosition;
	UINT32 endPosition;
	UINT32 startLine;
	UINT32 endLine;
	UINT32 startLinePosition;
	UINT32 endLinePosition;
	UINT32 totalSegments;
	UINT32 totalSpan;
	WINDOWLISTELEMENT* theWindowElement;
	EDITOR_WINDOW* theWindow;

	theWindowElement = (WINDOWLISTELEMENT*)theView->parentWindow->userData;
	if ((theWindow = theWindowElement->theEditorWindow) && theWindow->buffer)
	{
		NSRect wRect;

		wRect = theWindowElement->statusRef.bounds;
		// fill background
		CGContextSetRGBFillColor(cgContext, STATUS_BACK_COLOR_R, STATUS_BACK_COLOR_G, STATUS_BACK_COLOR_B, 1);
		CGContextFillRect(cgContext, CGRectMake(0, 0, wRect.size.width, wRect.size.height));

		CGContextSetRGBFillColor(cgContext, STATUS_LINE_COLOR_R, STATUS_LINE_COLOR_G, STATUS_LINE_COLOR_B, 1);
		CGContextFillRect(cgContext, CGRectMake(0, 0, 1, wRect.size.height));											// first vertical line
		CGContextFillRect(cgContext, CGRectMake(wRect.size.width - 1, 0, 1, wRect.size.height));							// last vertical line

		CGContextSelectFont(cgContext, statusBarFont->postScriptName, statusBarFont->size, kCGEncodingMacRoman);
		CGContextSetRGBFillColor(cgContext, STATUS_TEXT_COLOR_R, STATUS_TEXT_COLOR_G, STATUS_TEXT_COLOR_B, 1);

		if (theWindow->buffer->task)						// see if there is a task active in this window
		{
			CGContextShowTextAtPoint(cgContext, 10, STATUS_TEXT_Y_OFFSET + statusBarFont->ascent, (char*)RUNNING_TASK_STR, sizeof(RUNNING_TASK_STR) - 1);
		}
		else
		{
			CGContextShowTextAtPoint(cgContext, 10, STATUS_TEXT_Y_OFFSET + statusBarFont->ascent, (char*)IDLE_TASK_STR, sizeof(IDLE_TASK_STR) - 1);
		}

		CGContextSetRGBFillColor(cgContext, STATUS_LINE_COLOR_R, STATUS_LINE_COLOR_G, STATUS_LINE_COLOR_B, 1);
		CGContextFillRect(cgContext, CGRectMake(STATUS_DIRTY_TEXT_X_OFFSET - 5, 0, 1, wRect.size.height));				// second vertical line, after the task state


		// draw dirty info
		CGContextSetRGBFillColor(cgContext, STATUS_TEXT_COLOR_R, STATUS_TEXT_COLOR_G, STATUS_TEXT_COLOR_B, 1);
		if (AtUndoCleanPoint(theWindow->buffer))
		{
			CGContextShowTextAtPoint(cgContext, STATUS_DIRTY_TEXT_X_OFFSET, STATUS_TEXT_Y_OFFSET + statusBarFont->ascent, (char*)" ", 1);
		}
		else
		{
			CGContextShowTextAtPoint(cgContext, STATUS_DIRTY_TEXT_X_OFFSET, STATUS_TEXT_Y_OFFSET + statusBarFont->ascent, (char*)"*", 1);
		}

		CGContextSetRGBFillColor(cgContext, STATUS_LINE_COLOR_R, STATUS_LINE_COLOR_G, STATUS_LINE_COLOR_B, 1);
		CGContextFillRect(cgContext, CGRectMake(STATUS_POS_TEXT_X_OFFSET - 10, 0, 1, wRect.size.height));					// 3rd vertical line, asfter dirty


		// draw line info
		CGContextSetRGBFillColor(cgContext, STATUS_TEXT_COLOR_R, STATUS_TEXT_COLOR_G, STATUS_TEXT_COLOR_B, 1);
		EditorGetSelectionInfo(theWindow->buffer, theWindow->buffer->selectionUniverse, &startPosition, &endPosition, &startLine, &endLine, &startLinePosition, &endLinePosition, &totalSegments, &totalSpan);
		if (totalSegments)
		{
			sprintf(statusStr, "<Selections> Number:%d  TotalChars:%d  Start(%d:%d %d)  End(%d:%d %d)",
					totalSegments,
					totalSpan,
					startLine + 1, startLinePosition, startPosition,
					endLine + 1, endLinePosition, endPosition
				   );

		}
		else
		{
			sprintf(statusStr, "Lines:%d  Chars:%d  Cursor(%d:%d %d)",
					theWindow->buffer->textUniverse->totalLines,
					theWindow->buffer->textUniverse->totalBytes,
					startLine + 1, startLinePosition, startPosition
				   );
		}
		CGContextShowTextAtPoint(cgContext, STATUS_POS_TEXT_X_OFFSET, STATUS_TEXT_Y_OFFSET + statusBarFont->ascent, (char*)statusStr, strlen(statusStr));
	}
}


void UninitStatusBar()
//	Uninitialize status bar stuff
{
	FreeEditorFont(statusBarFont);
}

bool InitStatusBar()
//	Setup stuff needed for the status bar and bottom bar
{
	if ((statusBarFont = LoadEditorFont((const char*)STATUS_BAR_FONT_NAME)))
	{
		return (true);
	}
	return (false);
}

@implementation GUIStatusView
{
}

- (id)initWithFrame:(NSRect)frame
{
	self = [super initWithFrame:frame];
	if (self)
	{
	}
	return self;
}

- (BOOL)acceptsFirstResponder
{
	return NO;
}

- (BOOL)isFlipped
{
	return true;
}

- (BOOL)isOpaque
{
	return YES;
}


- (void)drawRect:(NSRect)rect
{
	WINDOWLISTELEMENT* theWindowElement;
	EDITOR_VIEW* theView;
	EDITOR_WINDOW* theWindow;

	theWindowElement = self.windowElement;
	theWindow = theWindowElement->theEditorWindow;
	theView = (EDITOR_VIEW*)theWindow->windowInfo;

	NSGraphicsContext* gc = [NSGraphicsContext currentContext];
	CGContextRef context = (CGContextRef)[gc graphicsPort];

	CGContextSetShouldAntialias(context, true);						// we want nice looking text
	CGContextSetAllowsAntialiasing(context, true);
	CGContextSetShouldSmoothFonts(context, true);
	CGContextSetTextMatrix(context, CGAffineTransformMake(1.0, 0.0, 0.0, -1.0, 0.0, 0.0));
	CGContextSetTextDrawingMode(context, kCGTextFill);

	DrawWindowStatusBar(theView, context);
}
@end
