// Copyright (C) 2009 Fred Allen.
//
// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

#include "includes.h"
#import <AppKit/AppKit.h>

#define	TOOLBAR_BACK_COLOR_R			0.93
#define	TOOLBAR_BACK_COLOR_G			0.93
#define	TOOLBAR_BACK_COLOR_B			0.93

#define	TOOLBAR_LINE_COLOR_R			0.5
#define	TOOLBAR_LINE_COLOR_G			0.5
#define	TOOLBAR_LINE_COLOR_B			0.5

#define	TOOLBAR_TEXT_COLOR_R			0.0
#define	TOOLBAR_TEXT_COLOR_G			0.0
#define	TOOLBAR_TEXT_COLOR_B			0.0

#define	BBAR_LINE_COLOR_DARK_R			0.31
#define	BBAR_LINE_COLOR_DARK_G			0.31
#define	BBAR_LINE_COLOR_DARK_B			0.31

#define	BBAR_LINE_COLOR_LIGHT_R			0.95
#define	BBAR_LINE_COLOR_LIGHT_G			0.95
#define	BBAR_LINE_COLOR_LIGHT_B			0.95

#define	BBAR_TEXT_COLOR_R				0.0
#define	BBAR_TEXT_COLOR_G				0.0
#define	BBAR_TEXT_COLOR_B				0.0

#define	BBAR_TEXT_DISABLED_COLOR_R		0.3
#define	BBAR_TEXT_DISABLED_COLOR_G		0.3
#define	BBAR_TEXT_DISABLED_COLOR_B		0.3

#define	BBAR_BOX_X_OFFSET				12							// X start of the first position box in the bar
#define	BBAR_BOX_Y_OFFSET				5							// Y offset of box in bar
#define	BBAR_INFO_BOX_W_OFFSET			16							// how much on the left and right extra inside the box is added for text info
#define	BBAR_BUTTON_BOX_W_OFFSET		10							// how much on the left and right extra inside the box is added for text button
#define	BBAR_BOX_HEIGHT					19							// height of a box
#define	BBAR_INFO_TEXT_X_OFFSET			8							// X offset inside box for text
#define	BBAR_BUTTON_TEXT_X_OFFSET		5							// X offset inside box for button
#define	BBAR_TEXT_Y_OFFSET				1							// Y offset inside box + font ascent
#define	BBAR_BOX_OFFSET_BETWEEN			16							// space between boxes

#define	MAX_TOOLBAR_POSITIONS			64

#define	TOOLBAR_FONT_NAME			"-*-Helvetica Neue-normal-r-normal--13--*-*-*-*-*-*"

static char localErrorFamily[] = "OSXgui";

enum
{
	TOOLBAR_POSITION_OUT_OF_RANGE
};

static const char* errorMembers[] =
{
	"StatusPositionOutOfRange"
};

static const char* errorDescriptions[] =
{
	"Field number for SetBottomBarPosition is out of range"
};

enum
{
	TOOLBAR_POSITION_TYPE_TEXT,
	TOOLBAR_POSITION_TYPE_BUTTON
};


typedef struct
{
	UINT32 type;
	UINT32 width;
	char* cmdString;
	char* string;
	bool selected;
} TOOLBAR_POSITION_ENTRY;

static TOOLBAR_POSITION_ENTRY theToolbarPositions[MAX_TOOLBAR_POSITIONS];
static EDITOR_FONT* toolBarFont;
static NSImage* bbarButtonN;
static NSImage* bbarButtonS;
static NSImage* bbarInfo;


static UINT32 GetStringWidth(EDITOR_FONT* theFont, const char* format, ...)
{
	UINT32 width;
	va_list args;
	char tempString[2048];
	int i;

	va_start(args, format);
	vsnprintf(tempString, sizeof(tempString), format, args);	// make the string
	va_end(args);

	width = 0;
	i = 0;
	while (tempString[i])
	{
		unsigned char theByte;

		theByte = (unsigned char)tempString[i];
		width += theFont->charWidths[theByte];
		i++;
	}
	return (width);
}

static INT32 DrawString(CGContextRef cgContext, EDITOR_FONT* theFont, INT32 x, INT32 y, const char* format, ...)
{
	va_list args;
	char tempString[2048];
	int i;

	va_start(args, format);
	vsnprintf(tempString, sizeof(tempString), format, args);	// make the string
	va_end(args);

	i = 0;
	while (tempString[i])
	{
		unsigned char theByte;

		theByte = (unsigned char)tempString[i];
		CGContextShowTextAtPoint(cgContext, x, y + theFont->ascent, (char*)&theByte, 1);
		x += theFont->charWidths[theByte];
		i++;
	}
	return (x);
}


@interface PopupMenuHandler : NSObject
	@property (nonatomic, assign) NSInteger retIndex;
@end

@implementation PopupMenuHandler
{
}

- (id)init
{
	if (self = [super init])
	{
	}
	return self;
}

- (void)handleMenuEvent:(NSMenuItem*)menu
{
	self.retIndex = menu.tag;
}
@end

bool CreatePopupMenuAndReturnItem(UINT32 argc, char* argv[], UINT32* item, bool* itemSelected)
// Passed in a list of items, create a popup menu of those items, display it at the current mouse position.
// If the selected something, set itemSelected to true and item to the item number. itemSelected gets set to false
// if the you closed the popup menu without selecting something (clicked away or hit escape).
// Return false if there was an error.
{
	NSMenu* popupMenu;
	UINT32 i;
	bool fail;

	(*itemSelected) = false;									// no item selected
	fail = false;
	if (argc)												// are there any items in the list
	{
		PopupMenuHandler* menuHandler = [[PopupMenuHandler alloc] init];

		popupMenu = [[NSMenu alloc] initWithTitle:@""];

		for (i = 0; !fail && (i < argc); i++)
		{
			NSMenuItem* menuItem = [[NSMenuItem alloc] initWithTitle:[NSString stringWithCString:argv[i] encoding:NSASCIIStringEncoding] action:nil keyEquivalent:@""];
			menuItem.enabled = YES;
			[menuItem setTag:i];
			[menuItem setTarget:menuHandler];
			[menuItem setAction:@selector(handleMenuEvent:)];
			[popupMenu insertItem:menuItem atIndex:i];
		}
		if (!fail)
		{
			if ([popupMenu popUpMenuPositioningItem:nil atLocation:[NSEvent mouseLocation] inView:nil])
			{
				(*itemSelected) = true;
				*item = (UINT32)menuHandler.retIndex;
			}
		}
	}
	return (!fail);
}

static bool SetGlobalBottomBarPosition(UINT32 position, UINT32 width, const char* theString, const char* cmdString, UINT32 type)
//	Set a global Toolbar position
{
	bool result;
	char* strPtr;

	result = false;
	if (position < MAX_TOOLBAR_POSITIONS)									// make sure it is in range
	{
		theToolbarPositions[position].type = type;
		if (width != 0)												// see if the position if going away
		{
			if ((strPtr = (char*)MNewPtr((UINT32) strlen(theString) + 1)))
			{
				strcpy(strPtr, theString);
				if (theToolbarPositions[position].string)
				{
					MDisposePtr(theToolbarPositions[position].string);
				}
				theToolbarPositions[position].string = strPtr;
				if (cmdString)
				{
					if ((strPtr = (char*)MNewPtr((UINT32)(strlen(cmdString) + 1))))
					{
						strcpy(strPtr, cmdString);
						if (theToolbarPositions[position].cmdString)
						{
							MDisposePtr(theToolbarPositions[position].cmdString);
						}
						theToolbarPositions[position].cmdString = strPtr;
						theToolbarPositions[position].width = width;
						InvalidateAllBottomBars();
						result = true;
					}
				}
				else
				{
					theToolbarPositions[position].width = width;
					InvalidateAllBottomBars();
					result = true;
				}
			}
		}
		else
		{
			theToolbarPositions[position].width = 0;						// set width to zero
			if (theToolbarPositions[position].string)						// did we have memory for a string?
			{
				MDisposePtr(theToolbarPositions[position].string);		// yes, get rid of it
				theToolbarPositions[position].string = NULL;
			}
			if (theToolbarPositions[position].cmdString)					// did we have memory for a string?
			{
				MDisposePtr(theToolbarPositions[position].cmdString);	// yes, get rid of it
				theToolbarPositions[position].cmdString = NULL;
			}
			InvalidateAllBottomBars();
			result = true;
		}
	}
	else
	{
		SetError(localErrorFamily, errorMembers[TOOLBAR_POSITION_OUT_OF_RANGE], errorDescriptions[TOOLBAR_POSITION_OUT_OF_RANGE]);
	}
	return (result);
}

UINT32 GetBottomBarStringWidth(const char* string)
// Passed a string, return the width of the string in pixels for the bottom bar.
{
	return (GetStringWidth(toolBarFont, string));
}

bool SetGlobalBottomBarText(UINT32 position, UINT32 width, const char* theString)
{
	return (SetGlobalBottomBarPosition(position, width, theString, NULL, TOOLBAR_POSITION_TYPE_TEXT));
}

bool SetGlobalBottomBarButton(UINT32 position, UINT32 width, const char* theString, const char* cmdString)
{
	return (SetGlobalBottomBarPosition(position, width, theString, cmdString, TOOLBAR_POSITION_TYPE_BUTTON));
}

void UninitToolBar()
//	Uninitialize tool bar stuff
{
	UINT32 i;

	for (i = 0; i < MAX_TOOLBAR_POSITIONS; i++)
	{
		theToolbarPositions[i].width = 0;
		if (theToolbarPositions[i].string)
		{
			MDisposePtr(theToolbarPositions[i].string);
			theToolbarPositions[i].string = NULL;
		}
		if (theToolbarPositions[i].cmdString)
		{
			MDisposePtr(theToolbarPositions[i].cmdString);
			theToolbarPositions[i].cmdString = NULL;
		}
	}
	FreeEditorFont(toolBarFont);
}

bool InitToolBar()
//	Setup stuff needed for the tool bar
{
	UINT32 i;

	if ((toolBarFont = LoadEditorFont((const char*)TOOLBAR_FONT_NAME)))
	{
		if ((bbarButtonN = [NSImage imageNamed:@"toolbarbutton_n"]))							// load out two buttons
		{
			if ((bbarButtonS = [NSImage imageNamed:@"toolbarbutton_s"]))
			{
				if ((bbarInfo = [NSImage imageNamed:@"toolbarinfo"]))							// and info text area
				{
					bbarInfo.capInsets = NSEdgeInsetsMake(9, 9, 9, 9);
					for (i = 0; i < MAX_TOOLBAR_POSITIONS; i++)
					{
						theToolbarPositions[i].width = 0;
						theToolbarPositions[i].string = NULL;
						theToolbarPositions[i].cmdString = NULL;
						theToolbarPositions[i].selected = false;
					}
					return (true);
				}
			}
		}
	}
	return (false);
}

static void CalcPositionRect(UINT32 position, EDITOR_RECT* areaRect)
// Pass in a position, calculate its rect
{
	UINT32 i;
	INT32 x;

	x = BBAR_BOX_X_OFFSET;
	for (i = 0; i < position; i++)
	{
		x = x + theToolbarPositions[i].width + BBAR_BOX_OFFSET_BETWEEN;
	}
	areaRect->x = x;
	areaRect->h = BBAR_BOX_HEIGHT;
	areaRect->y = (kToolBarHeight - BBAR_BOX_HEIGHT) / 2;
	switch (theToolbarPositions[i].type)
	{
		case TOOLBAR_POSITION_TYPE_TEXT:
		{
			areaRect->w = theToolbarPositions[i].width + BBAR_INFO_BOX_W_OFFSET;
			break;
		}
		case TOOLBAR_POSITION_TYPE_BUTTON:
		{
			areaRect->w = theToolbarPositions[i].width + BBAR_BUTTON_BOX_W_OFFSET;
			break;
		}
	}
}

void DrawToolBarPositions(WINDOWLISTELEMENT* theWindowElement, GUIToolView* toolbarView, CGContextRef cgContext, NSRect updateRect)
/*	Run through the list of global statuses, drawing each one that has a width greater than zero
 */
{
	EDITOR_RECT areaRect;
	UINT32 i;

	CGContextSelectFont(cgContext, toolBarFont->postScriptName, toolBarFont->size, kCGEncodingMacRoman);
	for (i = 0; i < MAX_TOOLBAR_POSITIONS; i++)
	{
		if (theToolbarPositions[i].width)
		{
			CalcPositionRect(i, &areaRect);
			switch (theToolbarPositions[i].type)
			{
				case TOOLBAR_POSITION_TYPE_TEXT:
				{
					[bbarInfo drawInRect:CGRectMake(areaRect.x, areaRect.y, areaRect.w, areaRect.h)];
					break;
				}
				case TOOLBAR_POSITION_TYPE_BUTTON:
				{
					if (theToolbarPositions[i].selected)
					{
						[bbarButtonS drawInRect:CGRectMake(areaRect.x, areaRect.y, areaRect.w, areaRect.h)];
					}
					else
					{
						[bbarButtonN drawInRect:CGRectMake(areaRect.x, areaRect.y, areaRect.w, areaRect.h)];
					}
					break;
				}
			}
			if (theWindowElement->windowRef.keyWindow)
			{
				CGContextSetRGBFillColor(cgContext, BBAR_TEXT_COLOR_R, BBAR_TEXT_COLOR_G, BBAR_TEXT_COLOR_B, 1);
			}
			else
			{
				CGContextSetRGBFillColor(cgContext, BBAR_TEXT_DISABLED_COLOR_R, BBAR_TEXT_DISABLED_COLOR_G, BBAR_TEXT_DISABLED_COLOR_B, 1);
			}
			DrawString(cgContext, toolBarFont, areaRect.x + BBAR_BUTTON_TEXT_X_OFFSET, areaRect.y + BBAR_TEXT_Y_OFFSET, theToolbarPositions[i].string);
		}
	}
}

static INT32 FindButton(INT32 x, INT32 y)
// Pass a x,y position, return which button the point is in, or -1 if non
{
	EDITOR_RECT areaRect;
	UINT32 i;

	for (i = 0; i < MAX_TOOLBAR_POSITIONS; i++)
	{
		if (theToolbarPositions[i].width && theToolbarPositions[i].type == TOOLBAR_POSITION_TYPE_BUTTON)
		{
			CalcPositionRect(i, &areaRect);
			if (PointInRECT(x, y, &areaRect))
			{
				return (i);
			}
		}
	}
	return (-1);
}

void HandleMouseDownEventInToolBar(NSEvent* theEvent, WINDOWLISTELEMENT* theWindowElement)
{
	INT32 position;
	NSPoint event_location = [theEvent locationInWindow];
	NSPoint local_point = [theWindowElement->toolBarRef convertPoint:event_location fromView:nil];

	if ((position = FindButton(local_point.x, local_point.y)) >= 0)
	{
		EDITOR_RECT areaRect;
		NSRect wRect;

		theToolbarPositions[position].selected = true;
		CalcPositionRect(position, &areaRect);
		wRect.origin.x = areaRect.x;
		wRect.origin.y = areaRect.y;
		wRect.size.width = areaRect.w;
		wRect.size.height = areaRect.h;
		[theWindowElement->toolBarRef setNeedsDisplayInRect:wRect];

		BOOL keepOn = YES;
		NSPoint mouseLoc = [theWindowElement->toolBarRef convertPoint:[theEvent locationInWindow] fromView:nil];
		while (keepOn)
		{
			theEvent = [NSApp
						nextEventMatchingMask:NSLeftMouseUpMask | NSLeftMouseDraggedMask | NSRightMouseUpMask | NSRightMouseDraggedMask | NSOtherMouseUpMask | NSOtherMouseDraggedMask
						untilDate:[[NSDate alloc] initWithTimeIntervalSinceNow:0.02]
						inMode:NSDefaultRunLoopMode
						dequeue:YES];

			switch ([theEvent type])
			{
				case NSRightMouseUp:
				case NSLeftMouseUp:
				{
					keepOn = NO;
					break;
				}
				case NSOtherMouseDragged:
				case NSRightMouseDragged:
				case NSLeftMouseDragged:
				{
					mouseLoc = [theWindowElement->toolBarRef convertPoint:[theEvent locationInWindow] fromView:nil];
				}
				default:
				{
					if (PointInRECT(mouseLoc.x, mouseLoc.y, &areaRect))
					{
						if (!theToolbarPositions[position].selected)
						{
							theToolbarPositions[position].selected = true;
							[theWindowElement->toolBarRef setNeedsDisplayInRect:wRect];
						}
					}
					else
					{
						if (theToolbarPositions[position].selected)
						{
							theToolbarPositions[position].selected = false;
							[theWindowElement->toolBarRef setNeedsDisplayInRect:wRect];
						}
					}
				}
			}
		}
		if (theToolbarPositions[position].selected)
		{
			ExecuteTclCommand(theToolbarPositions[position].cmdString);
		}
		theToolbarPositions[position].selected = false;
		InvalidateAllBottomBars();
	}
}


@implementation GUIToolView
{
}

- (id)initWithFrame:(NSRect)frame
{
	self = [super initWithFrame:frame];
	if (self)
	{
		[self registerForDraggedTypes:[NSArray arrayWithObjects:NSColorPboardType, NSFilenamesPboardType, nil]];
	}
	return self;
}

- (BOOL)isFlipped
{
	return true;
}

- (BOOL)isOpaque
{
	return YES;
}

- (void)mouseDown:(NSEvent*)theEvent
{
	WINDOWLISTELEMENT* theWindowElement;
	EDITOR_VIEW* theView;
	EDITOR_WINDOW* theWindow;

	theWindowElement = self.windowElement;
	theWindow = theWindowElement->theEditorWindow;
	HandleMouseDownEventInToolBar(theEvent, theWindowElement);
}

- (void)drawRect:(NSRect)rect
{
	WINDOWLISTELEMENT* theWindowElement;
	EDITOR_VIEW* theView;
	EDITOR_WINDOW* theWindow;

	theWindowElement = self.windowElement;
	theWindow = theWindowElement->theEditorWindow;

	NSGraphicsContext* gc = [NSGraphicsContext currentContext];
	CGContextRef context = (CGContextRef)[gc graphicsPort];

	CGContextSetShouldAntialias(context, true);						// we want nice looking text
	CGContextSetAllowsAntialiasing(context, true);
	CGContextSetShouldSmoothFonts(context, true);
	CGContextSetTextMatrix(context, CGAffineTransformMake(1.0, 0.0, 0.0, -1.0, 0.0, 0.0));
	CGContextSetTextDrawingMode(context, kCGTextFill);


	NSRect wRect;

	wRect = theWindowElement->toolBarRef.bounds;
	// fill background
	CGContextSetRGBFillColor(context, TOOLBAR_BACK_COLOR_R, TOOLBAR_BACK_COLOR_G, TOOLBAR_BACK_COLOR_B, 1);
	CGContextFillRect(context, CGRectMake(0, 0, wRect.size.width, wRect.size.height));

	CGContextSetRGBFillColor(context, TOOLBAR_LINE_COLOR_R, TOOLBAR_LINE_COLOR_G, TOOLBAR_LINE_COLOR_B, 1);
	CGContextFillRect(context, CGRectMake(0, 0, 1, wRect.size.height));												// first vertical line
	CGContextFillRect(context, CGRectMake(wRect.size.width - 1, 0, 1, wRect.size.height));							// last vertical line

	CGContextFillRect(context, CGRectMake(0, 0, wRect.size.width, 1));

	DrawToolBarPositions(theWindowElement, self, context, rect);
}

@end
