// Copyright (C) 2009 Fred Allen.
// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".


#define	EDIT_MAX_WIDTH				32767
#define	EDIT_MAX_HEIGHT				32767

#define	VIEW_TOP_MARGIN			4		// number of pixels to leave at the top of the view
#define	VIEW_BOTTOM_MARGIN		4		// number of pixels to leave at the bottom of the view
#define	VIEW_LEFT_MARGIN		10		// number of pixels to the left of the text in the view that we call the left hand side

typedef struct guiViewInfo
{
	EDITOR_RECT bounds;					// boundary of this view within its parent window
	EDITOR_RECT textBounds;				// roughly the bounds, but rounded down for the current font
	UINT32 topLine;						// current first line of edit universe that is displayed in the view
	INT32 leftPixel;					// current offset from left where text is being drawn in this view
	UINT32 lineHeight;					// number of pixels per line vertically
	UINT32 lineAscent;					// tells where the baseline of text is for each line in the view
	INT32 maxLeftOverhang;				// gives the maximum font character left overhang for any character in any style of this view
	INT32 maxRightOverhang;				// gives the maximum font character right overhang for any character in any style of this view
	INT32 maxCharWidth;					// gives the maximum font character width for any character in any style of this view
	UINT32 numLines;					// number of lines that can be displayed
	bool cursorOn;						// true if the cursor should be on at all
	bool oldCursorShowing;				// holds showing state of cursor across Start/EndCursorChange
	bool blinkState;					// tells what state (true = on) (false = off) the cursor blink is currently in
	char* titleString;
	EDITOR_COLOR foregroundColor;
	EDITOR_COLOR backgroundColor;
} GUI_VIEW_INFO;


void GetEditorViewGraphicToTextPosition(EDITOR_VIEW* view, UINT32 linePosition, INT32 xPosition, UINT32* betweenOffset, UINT32* charOffset);
void GetEditorViewTextToGraphicPosition(EDITOR_VIEW* view, UINT32 position, INT32* xPosition, bool limitMax, UINT32* slopLeft, UINT32* slopRight);
void GetEditorViewTextInfo(EDITOR_VIEW* view, UINT32* topLine, UINT32* numLines, INT32* leftPixel, UINT32* numPixels);
void RecalculateViewFontInfo(EDITOR_VIEW* view);
void SetViewBounds(EDITOR_VIEW* view, EDITOR_RECT* newBounds);
void InvalidateViewPortion(EDITOR_VIEW* view, UINT32 startLine, UINT32 endLine, UINT32 startPixel, UINT32 endPixel);
void InvalidateViewSelection(EDITOR_VIEW* view, SELECTION_UNIVERSE* selection);
void InvalidateView(EDITOR_VIEW* view);
void ScrollViewPortion(EDITOR_VIEW* view, UINT32 startLine, UINT32 endLine, INT32 numLines, UINT32 startPixel, UINT32 endPixel, INT32 numPixels);
void SetViewTopLeft(EDITOR_VIEW* view, UINT32 newTopLine, INT32 newLeftPixel);
void SetEditorViewTopLine(EDITOR_VIEW* view, UINT32 lineNumber);
void BlinkViewCursor(EDITOR_VIEW* theView);
void ResetEditorViewCursorBlink(EDITOR_VIEW* view);
void ViewCursorTask(EDITOR_VIEW* view);
void ViewStyleFontsChanged(EDITOR_VIEW* view);
void ViewEndChange(EDITOR_VIEW* view);
void ViewStartChange(EDITOR_VIEW* view);
void DisposeViewInfo(EDITOR_VIEW* view);
bool CreateViewInfo(EDITOR_VIEW* view, EDITOR_VIEW_DESCRIPTOR* descriptor);
void DrawView(EDITOR_VIEW* theView, void* cgContext, void* updateShape);
